import bpy
import bmesh
from rna_prop_ui import rna_idprop_ui_create
import bpy_extras 

class AddBranchedLightning(bpy.types.Operator):
    bl_label = "New Branched Lightning"
    bl_idname = 'lightning.branch'
    bl_options = {"REGISTER", "UNDO"}
    
    @classmethod
    def description(cls, context, properties):
        return "Adds a new 'Branched Lightning' at the position of the 3D cursor"
    
    def execute(self, context):
        
        scene = bpy.context.scene
        savePPoint = scene.tool_settings.transform_pivot_point
        scene.tool_settings.transform_pivot_point = 'MEDIAN_POINT'
        
        cursorLoc = bpy.context.scene.cursor.location
        current_frame = bpy.data.scenes[0].frame_current
        
        ui_Props = bpy.context.scene.randomL_props
        attachedObject = bpy.context.scene.attach_branch
        attachEnabled = ui_Props.branchL_enableAttachment
        

        #Check for already existing object with name--------------------------------------------------------------------------------------------------------
        def setUniqueName(newName):
            if bpy.data.objects.get(newName) is not None:
                checkName = newName
                counter = 1
                while bpy.data.objects.get(checkName) is not None:
                    counter += 1
                    checkName = newName + "_" + str(counter)
                return checkName
            else:
                return newName
        
        
        
        #Create Driver--------------------------------------------------------------------------------------------------------------------------------------
        def add_driver(source, prop, dataPath, name = '',func = ''):
            d = source.driver_add( prop ).driver
            v = d.variables.new()
            v.name                 = name
            v.targets[0].id        = source
            v.targets[0].data_path = dataPath
            d.expression = func
    
    
    
        #Add LightningMesh----------------------------------------------------------------------------------------------------------------------------------
        mesh = bpy.data.meshes.new("Vert")
        mesh.vertices.add(1)
        bpy_extras.object_utils.object_data_add(context, mesh, operator=None)
        bpy.ops.object.mode_set(mode='EDIT')
        bpy.ops.mesh.select_mode(use_extend=False, use_expand=False, type='VERT')

        
        lightningMeshName = setUniqueName("LightningMesh")
        bpy.context.object.name = lightningMeshName
        lightningMesh = bpy.data.objects[lightningMeshName]
        
        me = lightningMesh.data
        bm = bmesh.from_edit_mesh(me)
        
        bpy.ops.mesh.extrude_region_move(MESH_OT_extrude_region={}, TRANSFORM_OT_translate={"value":(0, 0, 1), "orient_type":'GLOBAL', "orient_matrix":((1, 0, 0), (0, 1, 0), (0, 0, 1))})
        bpy.ops.mesh.extrude_region_move(MESH_OT_extrude_region={}, TRANSFORM_OT_translate={"value":(0.5, 0.5, 1), "orient_type":'GLOBAL', "orient_matrix":((1, 0, 0), (0, 1, 0), (0, 0, 1))})
        bm.verts.ensure_lookup_table()
        bm.verts[2].select = False
        bm.verts[1].select = True
        bpy.ops.mesh.extrude_region_move(MESH_OT_extrude_region={}, TRANSFORM_OT_translate={"value":(-1, 0, 1), "orient_type":'GLOBAL', "orient_matrix":((1, 0, 0), (0, 1, 0), (0, 0, 1))})
        bm.verts.ensure_lookup_table()
        bm.verts[3].select = False
        bm.verts[2].select = True
        bpy.ops.mesh.extrude_region_move(MESH_OT_extrude_region={}, TRANSFORM_OT_translate={"value":(0, 1, 0.5), "orient_type":'GLOBAL', "orient_matrix":((1, 0, 0), (0, 1, 0), (0, 0, 1))})
        bm.verts.ensure_lookup_table()
        bm.verts[4].select = False
        bm.verts[2].select = True
        bpy.ops.mesh.extrude_region_move(MESH_OT_extrude_region={}, TRANSFORM_OT_translate={"value":(1, 0, 0.5), "orient_type":'GLOBAL', "orient_matrix":((1, 0, 0), (0, 1, 0), (0, 0, 1))})
        bm.verts.ensure_lookup_table()
        bm.verts[5].select = False
        bm.verts[3].select = True
        bpy.ops.mesh.extrude_region_move(MESH_OT_extrude_region={}, TRANSFORM_OT_translate={"value":(0.7, -0.3, 0.7), "orient_type":'GLOBAL', "orient_matrix":((1, 0, 0), (0, 1, 0), (0, 0, 1))})
        bm.verts.ensure_lookup_table()
        bm.verts[6].select = False
        bm.verts[3].select = True
        bpy.ops.mesh.extrude_region_move(MESH_OT_extrude_region={}, TRANSFORM_OT_translate={"value":(-0.8, -0.4, 0.5), "orient_type":'GLOBAL', "orient_matrix":((1, 0, 0), (0, 1, 0), (0, 0, 1))})
        bm.verts.ensure_lookup_table()
        bm.verts[7].select = False
        bm.verts[5].select = True
        bpy.ops.mesh.extrude_region_move(MESH_OT_extrude_region={}, TRANSFORM_OT_translate={"value":(0.4, -0.2, 0.5), "orient_type":'GLOBAL', "orient_matrix":((1, 0, 0), (0, 1, 0), (0, 0, 1))})
        bm.verts.ensure_lookup_table()
        bm.verts[8].select = False
        bm.verts[5].select = True
        bpy.ops.mesh.extrude_region_move(MESH_OT_extrude_region={}, TRANSFORM_OT_translate={"value":(0.4, 0.2, -0.5), "orient_type":'GLOBAL', "orient_matrix":((1, 0, 0), (0, 1, 0), (0, 0, 1))})
        bm.verts.ensure_lookup_table()
        bm.verts[9].select = False
        bm.verts[6].select = True
        bpy.ops.mesh.extrude_region_move(MESH_OT_extrude_region={}, TRANSFORM_OT_translate={"value":(0.4, 0.8, 0.5), "orient_type":'GLOBAL', "orient_matrix":((1, 0, 0), (0, 1, 0), (0, 0, 1))})
        bm.verts.ensure_lookup_table()
        bm.verts[10].select = False
        bm.verts[6].select = True
        bpy.ops.mesh.extrude_region_move(MESH_OT_extrude_region={}, TRANSFORM_OT_translate={"value":(0.6, -0.4, 0.7), "orient_type":'GLOBAL', "orient_matrix":((1, 0, 0), (0, 1, 0), (0, 0, 1))})
        bm.verts.ensure_lookup_table()
        bm.verts[11].select = False
        bm.verts[7].select = True
        bpy.ops.mesh.extrude_region_move(MESH_OT_extrude_region={}, TRANSFORM_OT_translate={"value":(-0.2, -0.6, 0.2), "orient_type":'GLOBAL', "orient_matrix":((1, 0, 0), (0, 1, 0), (0, 0, 1))})
        bm.verts.ensure_lookup_table()
        bm.verts[12].select = False
        bm.verts[7].select = True
        bpy.ops.mesh.extrude_region_move(MESH_OT_extrude_region={}, TRANSFORM_OT_translate={"value":(-0.4, 0.6, 0.1), "orient_type":'GLOBAL', "orient_matrix":((1, 0, 0), (0, 1, 0), (0, 0, 1))})
        bm.verts.ensure_lookup_table()
        bm.verts[13].select = False
        bm.verts[11].select = True
        bpy.ops.mesh.extrude_region_move(MESH_OT_extrude_region={}, TRANSFORM_OT_translate={"value":(0.4, -0.7, 0.7), "orient_type":'GLOBAL', "orient_matrix":((1, 0, 0), (0, 1, 0), (0, 0, 1))})
        bm.verts.ensure_lookup_table()
        bm.verts[14].select = False
        bm.verts[11].select = True
        bpy.ops.mesh.extrude_region_move(MESH_OT_extrude_region={}, TRANSFORM_OT_translate={"value":(0, 0.7, 1), "orient_type":'GLOBAL', "orient_matrix":((1, 0, 0), (0, 1, 0), (0, 0, 1))})
        bm.verts.ensure_lookup_table()
        bm.verts[15].select = False
        bm.verts[10].select = True
        bpy.ops.mesh.extrude_region_move(MESH_OT_extrude_region={}, TRANSFORM_OT_translate={"value":(0.4, 0.3, 0.1), "orient_type":'GLOBAL', "orient_matrix":((1, 0, 0), (0, 1, 0), (0, 0, 1))})
        bm.verts.ensure_lookup_table()
        bm.verts[16].select = False
        bm.verts[10].select = True
        bpy.ops.mesh.extrude_region_move(MESH_OT_extrude_region={}, TRANSFORM_OT_translate={"value":(-0.3, 0.8, 0.2), "orient_type":'GLOBAL', "orient_matrix":((1, 0, 0), (0, 1, 0), (0, 0, 1))})
        bm.verts.ensure_lookup_table()
        bm.verts[17].select = False
        bm.verts[15].select = True
        bpy.ops.mesh.extrude_region_move(MESH_OT_extrude_region={}, TRANSFORM_OT_translate={"value":(0.4, 0.6, 0.7), "orient_type":'GLOBAL', "orient_matrix":((1, 0, 0), (0, 1, 0), (0, 0, 1))})
        bm.verts.ensure_lookup_table()
        bm.verts[18].select = False
        bm.verts[15].select = True
        bpy.ops.mesh.extrude_region_move(MESH_OT_extrude_region={}, TRANSFORM_OT_translate={"value":(-0.4, -0.6, 0.9), "orient_type":'GLOBAL', "orient_matrix":((1, 0, 0), (0, 1, 0), (0, 0, 1))})
        bm.verts.ensure_lookup_table()
        bm.verts[19].select = False
        bm.verts[14].select = True
        bpy.ops.mesh.extrude_region_move(MESH_OT_extrude_region={}, TRANSFORM_OT_translate={"value":(-0.5, -0.6, 0.1), "orient_type":'GLOBAL', "orient_matrix":((1, 0, 0), (0, 1, 0), (0, 0, 1))})
        bm.verts.ensure_lookup_table()
        bm.verts[20].select = False
        bm.verts[14].select = True
        bpy.ops.mesh.extrude_region_move(MESH_OT_extrude_region={}, TRANSFORM_OT_translate={"value":(0.5, -0.3, 0.3), "orient_type":'GLOBAL', "orient_matrix":((1, 0, 0), (0, 1, 0), (0, 0, 1))})
        bm.verts.ensure_lookup_table()
        bm.verts[21].select = False
        bm.verts[19].select = True
        bpy.ops.mesh.extrude_region_move(MESH_OT_extrude_region={}, TRANSFORM_OT_translate={"value":(-0.4, -0.4, 0.7), "orient_type":'GLOBAL', "orient_matrix":((1, 0, 0), (0, 1, 0), (0, 0, 1))})
        bm.verts.ensure_lookup_table()
        bm.verts[22].select = False
        bm.verts[19].select = True
        bpy.ops.mesh.extrude_region_move(MESH_OT_extrude_region={}, TRANSFORM_OT_translate={"value":(-0.4, 0.6, 0.4), "orient_type":'GLOBAL', "orient_matrix":((1, 0, 0), (0, 1, 0), (0, 0, 1))})
        
        bpy.ops.mesh.select_all(action='SELECT')
        bpy.ops.transform.resize(value=(0.5, 0.5, 0.5))
        bpy.ops.transform.translate(value=(0, 0, -1.625))
        
        bpy.ops.mesh.select_all(action='DESELECT')
        bm.verts.ensure_lookup_table()
        bm.verts[0].select = True
        bpy.ops.view3d.snap_selected_to_cursor(use_offset=False)

        bpy.ops.object.editmode_toggle()
        bpy.context.object.lock_location[0] = True
        bpy.context.object.lock_location[1] = True
        bpy.context.object.lock_location[2] = True
        bpy.context.object.lock_rotation[0] = True
        bpy.context.object.lock_rotation[1] = True
        bpy.context.object.lock_rotation[2] = True
        
        
        #Vertex Groups
        displInfluence = lightningMesh.vertex_groups.new(name = 'Displacement Influence')
        displInfluence.add((0, 0), 1, 'REPLACE')
        displInfluence.add((1, 1), 0.3, 'REPLACE')
        
        thicknessRoot = lightningMesh.vertex_groups.new(name = 'Root Thickness')
        thicknessRoot.add((0, 0), 1, 'REPLACE')
        thicknessRoot.add((1, 1), 0.5, 'REPLACE')
        thicknessRoot.add((2, 3), 0.2, 'REPLACE')
        
        thicknessMiddle = lightningMesh.vertex_groups.new(name = 'Middle Thickness')
        thicknessMiddle.add((1, 2, 3, 5, 6, 7, 10, 11, 14, 15, 19), 1, 'REPLACE')
        
        
        
        #Add LightningRoot----------------------------------------------------------------------------------------------------------------------------------
        bpy.ops.object.empty_add(type='CIRCLE', align='WORLD', location=lightningMesh.location, scale=(1, 1, 1))
        lightningRootName = setUniqueName("BranchedLightning_Root")
        bpy.context.object.name = lightningRootName
        lightningRoot = bpy.data.objects[lightningRootName]
        bpy.ops.transform.rotate(value=1.5708, orient_axis='X', orient_type='GLOBAL', orient_matrix=((1, 0, 0), (0, 1, 0), (0, 0, 1)))
        bpy.ops.transform.resize(value=(0.4, 0.4, 0.4), orient_type='GLOBAL', orient_matrix=((1, 0, 0), (0, 1, 0), (0, 0, 1)))

        bpy.ops.object.empty_add(type='CIRCLE', align='WORLD', location=lightningMesh.location, scale=(1, 1, 1))
        lightningMovName = setUniqueName("LightningMovement")
        bpy.context.object.name = lightningMovName
        lightningMovement = bpy.data.objects[lightningMovName]
        bpy.ops.transform.rotate(value=1.5708, orient_axis='X', orient_type='GLOBAL', orient_matrix=((1, 0, 0), (0, 1, 0), (0, 0, 1)))
        bpy.ops.transform.resize(value=(0.3, 0.3, 0.3), orient_type='GLOBAL', orient_matrix=((1, 0, 0), (0, 1, 0), (0, 0, 1)))

        #Make Root Parent of Mesh and Movement-Object
        lightningMesh.parent = lightningRoot
        lightningMesh.matrix_parent_inverse = lightningRoot.matrix_world.inverted()

        lightningMovement.parent = lightningRoot
        lightningMovement.matrix_parent_inverse = lightningRoot.matrix_world.inverted()



        #Add Lightning Handles------------------------------------------------------------------------------------------------------------------------------
        #Add Hook 1
        bpy.ops.object.empty_add(type='SPHERE', align='WORLD', location=cursorLoc, scale=(1, 1, 1))
        bpy.ops.transform.resize(value=(0.1, 0.1, 0.1), orient_type='GLOBAL', orient_matrix=((1, 0, 0), (0, 1, 0), (0, 0, 1)))
        controlName1 = setUniqueName("LightningCTRL_1")
        bpy.context.object.name = controlName1
        lightningCTRL_1 = bpy.data.objects[controlName1]
        bpy.ops.transform.translate(value=(0.98542, 0.20417, 1.25), orient_type='GLOBAL', orient_matrix=((1, 0, 0), (0, 1, 0), (0, 0, 1)))
        
        hookInf1 = lightningMesh.vertex_groups.new(name = 'Hook 1 Influence')
        hookInf1.add((8, 9), 1, 'REPLACE')
        hookInf1.add((5, 5), 0.6, 'REPLACE')
        hookInf1.add((2, 2), 0.5, 'REPLACE')
        hookInf1.add((4, 4), 0.55, 'REPLACE')
        
        modifier1 = lightningMesh.modifiers.new("Hook1", 'HOOK')
        modifier1.object = lightningCTRL_1
        modifier1.vertex_indices_set([8, 9, 5, 2, 4])
        modifier1.show_expanded = False
        modifier1.vertex_group = "Hook 1 Influence"
        
        lightningCTRL_1.parent = lightningRoot
        lightningCTRL_1.matrix_parent_inverse = lightningRoot.matrix_world.inverted()
        
        
        #Add Hook 2
        bpy.ops.object.empty_add(type='SPHERE', align='WORLD', location=cursorLoc, scale=(1, 1, 1))
        bpy.ops.transform.resize(value=(0.1, 0.1, 0.1), orient_type='GLOBAL', orient_matrix=((1, 0, 0), (0, 1, 0), (0, 0, 1)))
        controlName2 = setUniqueName("LightningCTRL_2")
        bpy.context.object.name = controlName2
        lightningCTRL_2 = bpy.data.objects[controlName2]
        bpy.ops.transform.translate(value=(-0.46458, -0.045833, 1), orient_type='GLOBAL', orient_matrix=((1, 0, 0), (0, 1, 0), (0, 0, 1)))
        
        hookInf2 = lightningMesh.vertex_groups.new(name = 'Hook 2 Influence')
        hookInf2.add((3, 3), 1, 'REPLACE')
        hookInf2.add((6, 6), 0.5, 'REPLACE')
        
        modifier2 = lightningMesh.modifiers.new("Hook2", 'HOOK')
        modifier2.object = lightningCTRL_2
        modifier2.vertex_indices_set([3, 6])
        modifier2.show_expanded = False
        modifier2.vertex_group = "Hook 2 Influence"
        
        lightningCTRL_2.parent = lightningRoot
        lightningCTRL_2.matrix_parent_inverse = lightningRoot.matrix_world.inverted()
        
        
        #Add Hook 3
        bpy.ops.object.empty_add(type='SPHERE', align='WORLD', location=cursorLoc, scale=(1, 1, 1))
        bpy.ops.transform.resize(value=(0.1, 0.1, 0.1), orient_type='GLOBAL', orient_matrix=((1, 0, 0), (0, 1, 0), (0, 0, 1)))
        controlName3 = setUniqueName("LightningCTRL_3")
        bpy.context.object.name = controlName3
        lightningCTRL_3 = bpy.data.objects[controlName3]
        bpy.ops.transform.translate(value=(-1.0146, -0.24583, 1.325), orient_type='GLOBAL', orient_matrix=((1, 0, 0), (0, 1, 0), (0, 0, 1)))
        
        hookInf3 = lightningMesh.vertex_groups.new(name = 'Hook 3 Influence')
        hookInf3.add((12, 13), 1, 'REPLACE')
        hookInf3.add((7, 7), 0.6, 'REPLACE')
        
        modifier3 = lightningMesh.modifiers.new("Hook3", 'HOOK')
        modifier3.object = lightningCTRL_3
        modifier3.vertex_indices_set([13, 12, 7])
        modifier3.show_expanded = False
        modifier3.vertex_group = "Hook 3 Influence"
        
        lightningCTRL_3.parent = lightningRoot
        lightningCTRL_3.matrix_parent_inverse = lightningRoot.matrix_world.inverted()
        
        
        #Add Hook 4
        bpy.ops.object.empty_add(type='SPHERE', align='WORLD', location=cursorLoc, scale=(1, 1, 1))
        bpy.ops.transform.resize(value=(0.1, 0.1, 0.1), orient_type='GLOBAL', orient_matrix=((1, 0, 0), (0, 1, 0), (0, 0, 1)))
        controlName4 = setUniqueName("LightningCTRL_4")
        bpy.context.object.name = controlName4
        lightningCTRL_4 = bpy.data.objects[controlName4]
        bpy.ops.transform.translate(value=(0.18542, -0.39583, 1.7), orient_type='GLOBAL', orient_matrix=((1, 0, 0), (0, 1, 0), (0, 0, 1)))
        
        hookInf4 = lightningMesh.vertex_groups.new(name = 'Hook 4 Influence')
        hookInf4.add((11, 11), 1, 'REPLACE')
        hookInf4.add((6, 6), 0.5, 'REPLACE')
        
        modifier4 = lightningMesh.modifiers.new("Hook4", 'HOOK')
        modifier4.object = lightningCTRL_4
        modifier4.vertex_indices_set([6, 11])
        modifier4.show_expanded = False
        modifier4.vertex_group = "Hook 4 Influence"
        
        lightningCTRL_4.parent = lightningRoot
        lightningCTRL_4.matrix_parent_inverse = lightningRoot.matrix_world.inverted()
        
        
        #Add Hook 5
        bpy.ops.object.empty_add(type='SPHERE', align='WORLD', location=cursorLoc, scale=(1, 1, 1))
        bpy.ops.transform.resize(value=(0.1, 0.1, 0.1), orient_type='GLOBAL', orient_matrix=((1, 0, 0), (0, 1, 0), (0, 0, 1)))
        controlName5 = setUniqueName("LightningCTRL_5")
        bpy.context.object.name = controlName5
        lightningCTRL_5 = bpy.data.objects[controlName5]
        bpy.ops.transform.translate(value=(0.11042, 0.47917, 1.675), orient_type='GLOBAL', orient_matrix=((1, 0, 0), (0, 1, 0), (0, 0, 1)))
        
        hookInf5 = lightningMesh.vertex_groups.new(name = 'Hook 5 Influence')
        hookInf5.add((16, 17), 1, 'REPLACE')
        hookInf5.add((10, 10), 0.6, 'REPLACE')
        
        modifier5 = lightningMesh.modifiers.new("Hook5", 'HOOK')
        modifier5.object = lightningCTRL_5
        modifier5.vertex_indices_set([16, 17, 10])
        modifier5.show_expanded = False
        modifier5.vertex_group = "Hook 5 Influence"
        
        lightningCTRL_5.parent = lightningRoot
        lightningCTRL_5.matrix_parent_inverse = lightningRoot.matrix_world.inverted()
        
        
        #Add Hook 6
        bpy.ops.object.empty_add(type='SPHERE', align='WORLD', location=cursorLoc, scale=(1, 1, 1))
        bpy.ops.transform.resize(value=(0.1, 0.1, 0.1), orient_type='GLOBAL', orient_matrix=((1, 0, 0), (0, 1, 0), (0, 0, 1)))
        controlName6 = setUniqueName("LightningCTRL_6")
        bpy.context.object.name = controlName6
        lightningCTRL_6 = bpy.data.objects[controlName6]
        bpy.ops.transform.translate(value=(0.38542, -0.97083, 2.15), orient_type='GLOBAL', orient_matrix=((1, 0, 0), (0, 1, 0), (0, 0, 1)))
        
        hookInf6 = lightningMesh.vertex_groups.new(name = 'Hook 6 Influence')
        hookInf6.add((20, 21), 1, 'REPLACE')
        hookInf6.add((14, 14), 0.6, 'REPLACE')
        
        modifier6 = lightningMesh.modifiers.new("Hook6", 'HOOK')
        modifier6.object = lightningCTRL_6
        modifier6.vertex_indices_set([20, 21, 14])
        modifier6.show_expanded = False
        modifier6.vertex_group = "Hook 6 Influence"
        
        lightningCTRL_6.parent = lightningRoot
        lightningCTRL_6.matrix_parent_inverse = lightningRoot.matrix_world.inverted()
        
        
        #Add Hook 7
        bpy.ops.object.empty_add(type='SPHERE', align='WORLD', location=cursorLoc, scale=(1, 1, 1))
        bpy.ops.transform.resize(value=(0.1, 0.1, 0.1), orient_type='GLOBAL', orient_matrix=((1, 0, 0), (0, 1, 0), (0, 0, 1)))
        controlName7 = setUniqueName("LightningCTRL_7")
        bpy.context.object.name = controlName7
        lightningCTRL_7 = bpy.data.objects[controlName7]
        bpy.ops.transform.translate(value=(-0.21458, -0.29583, 2.925), orient_type='GLOBAL', orient_matrix=((1, 0, 0), (0, 1, 0), (0, 0, 1)))
        
        hookInf7 = lightningMesh.vertex_groups.new(name = 'Hook 7 Influence')
        hookInf7.add((22, 23), 1, 'REPLACE')
        hookInf7.add((19, 19), 0.65, 'REPLACE')
        hookInf7.add((15, 15), 0.4, 'REPLACE')
        hookInf7.add((18, 18), 0.6, 'REPLACE')
        
        modifier7 = lightningMesh.modifiers.new("Hook7", 'HOOK')
        modifier7.object = lightningCTRL_7
        modifier7.vertex_indices_set([22, 23, 19, 18, 15])
        modifier7.show_expanded = False
        modifier7.vertex_group = "Hook 7 Influence"
        
        lightningCTRL_7.parent = lightningRoot
        lightningCTRL_7.matrix_parent_inverse = lightningRoot.matrix_world.inverted()
        
        
        
        
        #Adding Effect Modifiers to LighningMesh------------------------------------------------------------------------------------------------------------

        #Subdividion Modifier 1 (Detail):
        subDModifier_1 = lightningMesh.modifiers.new('Detail Subdivision', 'SUBSURF')
        subDModifier_1.levels = 5
        subDModifier_1.render_levels = 5
        subDModifier_1.show_expanded = False


        #Displace X:
        displaceX = lightningMesh.modifiers.new('Displace X', 'DISPLACE')
        displaceX.vertex_group = "Displacement Influence"
        displaceX.strength = 1
        displaceX.invert_vertex_group = True
        displaceX.texture_coords = 'OBJECT'
        displaceX.texture_coords_object = lightningMovement
        displaceX.direction = 'X'

        xDisplTex = bpy.data.textures.new('DisplacementTextureX', type = 'CLOUDS')
        xDisplTex.noise_scale = 3
        xDisplTex.noise_depth = 3
        xDisplTex.noise_basis = 'BLENDER_ORIGINAL'
        displaceX.texture = xDisplTex

        displaceX.show_expanded = False


        #Displace Y:
        displaceY = lightningMesh.modifiers.new('Displace Y', 'DISPLACE')
        displaceY.vertex_group = "Displacement Influence"
        displaceY.strength = 1
        displaceY.invert_vertex_group = True
        displaceY.texture_coords = 'OBJECT'
        displaceY.texture_coords_object = lightningMovement
        displaceY.direction = 'Y'

        yDisplTex = bpy.data.textures.new('DisplacementTextureY', type = 'CLOUDS')
        yDisplTex.noise_scale = 3
        yDisplTex.noise_depth = 3
        yDisplTex.noise_basis = 'VORONOI_F1'
        displaceY.texture = yDisplTex

        displaceY.show_expanded = False


        #Displace Z:
        displaceZ = lightningMesh.modifiers.new('Displace Z', 'DISPLACE')
        displaceZ.vertex_group = "Displacement Influence"
        displaceZ.strength = 1
        displaceZ.invert_vertex_group = True
        displaceZ.texture_coords = 'OBJECT'
        displaceZ.texture_coords_object = lightningMovement
        displaceZ.direction = 'Z'

        zDisplTex = bpy.data.textures.new('DisplacementTextureZ', type = 'CLOUDS')
        zDisplTex.noise_scale = 3
        zDisplTex.noise_depth = 3
        zDisplTex.noise_basis = 'ORIGINAL_PERLIN'
        displaceZ.texture = zDisplTex

        displaceZ.show_expanded = False


        #Build Start:
        buildStartMod = lightningMesh.modifiers.new('Start', 'BUILD')
        buildStartMod.frame_start = 1
        buildStartMod.frame_duration = 5
        buildStartMod.show_expanded = False


        #Build Start:
        buildStopMod = lightningMesh.modifiers.new('Stop', 'BUILD')
        buildStopMod.frame_start = 50
        buildStopMod.frame_duration = 5
        buildStopMod.use_reverse = True
        buildStopMod.show_expanded = False


        #Subdividion Modifier 2 (Smoothing):
        subDModifier_2 = lightningMesh.modifiers.new('Smoothing Subdivision', 'SUBSURF')
        subDModifier_2.levels = 0
        subDModifier_2.render_levels = 0
        subDModifier_2.show_expanded = False


        #Skin Modifier
        skinModifier = lightningMesh.modifiers.new('Base Thickness', 'SKIN')
        skinModifier.use_x_symmetry = False
        skinModifier.show_expanded = False
        skinModifier.use_smooth_shade = True

        bpy.ops.object.select_all(action="DESELECT")
        lightningMesh.select_set(True)
        bpy.context.view_layer.objects.active = lightningMesh
        bpy.ops.object.editmode_toggle()
        bpy.ops.mesh.select_all(action='SELECT')
        bpy.ops.transform.skin_resize(value=(0.05, 0.05, 0.05), orient_type='GLOBAL', orient_matrix=((1, 0, 0), (0, 1, 0), (0, 0, 1)))

        bpy.ops.mesh.select_all(action='DESELECT')
        bm = bmesh.from_edit_mesh(me)
        bm.verts.ensure_lookup_table()
        bm.verts[4].select = True
        bm.verts[8].select = True
        bm.verts[9].select = True
        bm.verts[13].select = True
        bm.verts[12].select = True
        bm.verts[17].select = True
        bm.verts[16].select = True
        bm.verts[18].select = True
        bm.verts[23].select = True
        bm.verts[22].select = True
        bm.verts[20].select = True
        bm.verts[21].select = True
        bpy.ops.transform.skin_resize(value=(0.02, 0.02, 0.02), orient_type='GLOBAL', orient_matrix=((1, 0, 0), (0, 1, 0), (0, 0, 1)))
        bpy.ops.object.editmode_toggle()
        
        
        #Tickness Middle:
        displaceMiddle = lightningMesh.modifiers.new('Thickness Middle', 'DISPLACE')
        displaceMiddle.vertex_group = "Middle Thickness"
        displaceMiddle.texture_coords = 'LOCAL'
        displaceMiddle.direction = 'NORMAL'
        displaceMiddle.show_expanded = False
        
        
        #Tickness Root:
        displaceRoot = lightningMesh.modifiers.new('Thickness Root', 'DISPLACE')
        displaceRoot.vertex_group = "Root Thickness"
        displaceRoot.texture_coords = 'LOCAL'
        displaceRoot.direction = 'NORMAL'
        displaceRoot.show_expanded = False
        



        #Create and/or Assign Material----------------------------------------------------------------------------------------------------------------------
        mat = bpy.data.materials.get("Lightning Material")
        if mat is None:
            mat = bpy.data.materials.new(name="Lightning Material")

        mat.use_nodes = True
        nodes = mat.node_tree.nodes
        nodes.clear()
        node_emission = nodes.new(type='ShaderNodeEmission')
        node_emission.inputs[0].default_value = (0.121746,0.260608,1,1)     #Color
        node_emission.inputs[1].default_value = 20                          # strength
        node_emission.location = 0,0

        node_output = nodes.new(type='ShaderNodeOutputMaterial')   
        node_output.location = 200,0
        links = mat.node_tree.links
        link = links.new(node_emission.outputs[0], node_output.inputs[0])

        node_ramp = nodes.new(type='ShaderNodeValToRGB')   
        node_ramp.location = -300,0
        link = links.new(node_ramp.outputs[0], node_emission.inputs[0])
        node_ramp.color_ramp.elements[0].color = (0.127438, 0.386429, 1, 1)
        node_ramp.color_ramp.elements[1].color = (0.391573, 0.061246, 1, 1)

        node_layerWeight = nodes.new(type='ShaderNodeLayerWeight')  
        node_layerWeight.location = -500,0
        link = links.new(node_layerWeight.outputs[1], node_ramp.inputs[0])
        node_layerWeight.inputs[0].default_value = 0.8

        lightningMesh.data.materials.append(mat)        
        
        
        
        #Create custom properties with drivers--------------------------------------------------------------------------------------------------------------

        #Detail Subdivision
        lightningMesh["Lightning Detail"] = 5
        lnDetail_Prop = rna_idprop_ui_create(lightningMesh, 'Lightning Detail', default = 5, min = 0, max = 6, soft_min = 0, soft_max = 6, description = 'The resolution used for displacement')
        add_driver(lightningMesh, 'modifiers["Detail Subdivision"].levels', '["Lightning Detail"]', 'Lightning_Detail', 'Lightning_Detail')
        add_driver(lightningMesh, 'modifiers["Detail Subdivision"].render_levels', '["Lightning Detail"]', 'Lightning_Detail', 'Lightning_Detail')
        
        #Smoothing Subdivision
        lightningMesh["Lightning Smoothing"] = 0
        lnSmooth_Prop = rna_idprop_ui_create(lightningMesh, 'Lightning Smoothing', default = 0, min = 0, max = 6, soft_min = 0, soft_max = 6, description = 'Smoothing of the lightning')
        add_driver(lightningMesh, 'modifiers["Smoothing Subdivision"].levels', '["Lightning Smoothing"]', 'Lightning_Smoothing', 'Lightning_Smoothing')
        add_driver(lightningMesh, 'modifiers["Smoothing Subdivision"].render_levels', '["Lightning Smoothing"]', 'Lightning_Smoothing', 'Lightning_Smoothing')
        
        #Start Time
        lightningMesh["Start Time"] = 0
        lnStart_Prop = rna_idprop_ui_create(lightningMesh, 'Start Time', default = current_frame - 5, min = -999999999, max = 999999999, description = 'The time when the lightning starts to develop')
        add_driver(lightningMesh, 'modifiers["Start"].frame_start', '["Start Time"]', 'Start_Time', 'Start_Time')
        
        #Stop Time
        lightningMesh["Stop Time"] = 0
        lnStop_Prop = rna_idprop_ui_create(lightningMesh, 'Stop Time', default = current_frame + 50, min = -999999999, max = 999999999, description = 'The time when the lightning starts to disappear')
        add_driver(lightningMesh, 'modifiers["Stop"].frame_start', '["Stop Time"]', 'Stop_Time', 'Stop_Time')
        
        #Start Duration
        lightningMesh["Start Duration"] = 5
        lnStartDur_Prop = rna_idprop_ui_create(lightningMesh, 'Start Duration', default = 5, min = 0, max = 999999999, soft_min = 0, description = 'The time it takes for the lightning to develop')
        add_driver(lightningMesh, 'modifiers["Start"].frame_duration', '["Start Duration"]', 'Start_Duration', 'Start_Duration')
        
        #Stop Duration
        lightningMesh["Stop Duration"] = 5
        lnStopDur_Prop = rna_idprop_ui_create(lightningMesh, 'Stop Duration', default = 5, min = 0, max = 999999999, soft_min = 0, description = 'The time it takes for the lightning to disappear')
        add_driver(lightningMesh, 'modifiers["Stop"].frame_duration', '["Stop Duration"]', 'Stop_Duration', 'Stop_Duration')
        
        #Distortion Strength
        lightningMesh["Distortion Strength"] = 1.5
        lnDistStr_Prop = rna_idprop_ui_create(lightningMesh, 'Distortion Strength', default = 1.5, min = 0.0, max = 999999999.0, soft_min = 0.0, description = 'Strength of the distortion of the lightning')
        add_driver(lightningMesh, 'modifiers["Displace X"].strength', '["Distortion Strength"]', 'Distortion_Strength', 'Distortion_Strength')
        add_driver(lightningMesh, 'modifiers["Displace Y"].strength', '["Distortion Strength"]', 'Distortion_Strength', 'Distortion_Strength')
        add_driver(lightningMesh, 'modifiers["Displace Z"].strength', '["Distortion Strength"]', 'Distortion_Strength', 'Distortion_Strength')
        
        #Thickness Root
        lightningMesh["Thickness Root"] = 0.1
        lnThickRoot_Prop = rna_idprop_ui_create(lightningMesh, 'Thickness Root', default = 0.1, min = 0.0, max = 1.0, soft_min = 0, soft_max = 1.0, description = 'The thickness of the lightning at the root')
        add_driver(lightningMesh, 'modifiers["Thickness Root"].strength', '["Thickness Root"]', 'Thickness_Root', 'Thickness_Root /2')
        
        #Thickness Middle
        lightningMesh["Thickness Middle"] = 0.05
        lnThickMid_Prop = rna_idprop_ui_create(lightningMesh, 'Thickness Middle', default = 0.05, min = 0.0, max = 1.0, soft_min = 0.0, soft_max = 1.0, description = 'The thickness of the lightning in the middle')
        add_driver(lightningMesh, 'modifiers["Thickness Middle"].strength', '["Thickness Middle"]', 'Thickness_Middle', '(Thickness_Middle /2)-0.03')
            
        scene.tool_settings.transform_pivot_point = savePPoint
        return {'FINISHED'}
    
    
def register() :
    bpy.utils.register_class(AddBranchedLightning)
 
def unregister() :
    bpy.utils.unregister_class(AddBranchedLightning)