import bpy
import sys
import bmesh
import random
import mathutils
from rna_prop_ui import rna_idprop_ui_create
from bpy_extras import object_utils

#Get Module of other Class
#ui_Module = sys.modules[modulesNames['electrify_ui']]
 
class AddRandomLightning(bpy.types.Operator):
    bl_label = "New Random Branched Lightning"
    bl_idname = 'lightning.random'
    bl_options = {"REGISTER", "UNDO"}
    
    
    @classmethod
    def description(cls, context, properties):
        return "Adds a new 'Random Lightning' at the position\nof the 3D cursor using the values above"
    
    
    def execute(self, context):        

        scene = bpy.context.scene
        savePPoint = scene.tool_settings.transform_pivot_point
        scene.tool_settings.transform_pivot_point = 'MEDIAN_POINT'
        cursorLoc = bpy.context.scene.cursor.location
        current_frame = bpy.data.scenes[0].frame_current
        ui_Props = bpy.context.scene.randomL_props
        attachedObject = bpy.context.scene.attach_rand
        attachEnabled = ui_Props.randL_enableAttachment


        vertBranching = {}
        availableVerts = []
        tipVerts = []
        middleVerts = []
        
        
        branchAmount = ui_Props.randL_BranchAmount
        maxBranchPerPoint = ui_Props.randL_MaxBranchPV
        limit_nX = ui_Props.randL_nXLimit
        limit_pX = ui_Props.randL_pXLimit
        limit_nY = ui_Props.randL_nYLimit
        limit_pY = ui_Props.randL_pYLimit
        limit_nZ = ui_Props.randL_nZLimit
        limit_pZ = ui_Props.randL_pZLimit
        
        #Reset UI Object Picker
        bpy.context.scene.attach_rand = None
    
        #Check for already existing object with name--------------------------------------------------------------------------------------------------------
        def setUniqueName(newName):
            if bpy.data.objects.get(newName) is not None:
                checkName = newName
                counter = 1
                while bpy.data.objects.get(checkName) is not None:
                    counter += 1
                    checkName = newName + "_" + str(counter)
                return checkName
            else:
                return newName
            
            
        #Create Driver--------------------------------------------------------------------------------------------------------------------------------------
        def add_driver(source, prop, dataPath, name = '',func = ''):
            d = source.driver_add( prop ).driver

            v = d.variables.new()
            v.name                 = name
            v.targets[0].id        = source
            v.targets[0].data_path = dataPath
            d.expression = func 
            
        def add_driverAO(source, propObj, prop, dataPath, name = '',func = ''):
            d = source.driver_add( prop ).driver

            v = d.variables.new()
            v.name                 = name
            v.targets[0].id        = propObj
            v.targets[0].data_path = dataPath
            d.expression = func 

            
        def randLocInRange():
            randomX = random.uniform(limit_nX, limit_pX)
            randomY = random.uniform(limit_nY, limit_pY)
            randomZ = random.uniform(limit_nZ, limit_pZ) 
            newLoc = mathutils.Vector((randomX, randomY, randomZ))
            return newLoc
        
        
        #Vertex Groups with distance to attached target object
        def vertGroupByDistance(obj, target):
            
            attachedDisplInfl = lightningMesh.vertex_groups.new(name = 'Displacement Influence With Attachment')
            attachedDisplInfl.add(tipVerts, 1, 'REPLACE')
            attachedDisplInfl.add((0, 0), 1, 'REPLACE')
            attachedDisplInfl.add((1, 1), 0.3, 'REPLACE')
            
            obj.select_set(True)
            bpy.ops.object.editmode_toggle()
            me = obj.data
            bm = bmesh.from_edit_mesh(me)
            maxDist = -99999999.0
            minDist = 99999999.0
            attachmentInfl = lightningMesh.vertex_groups.new(name = 'Attachment Influence')
            verts = []
            vertWeight = {}
            
            #Get distance of every Vertex to the target object and get min/max distance
            for vert in bm.verts:
                distance = (attachedObject.matrix_world.translation - (obj.matrix_world @ vert.co)).length
                
                if distance > maxDist:
                    maxDist = distance
                
                if distance < minDist:
                    minDist = distance
            
            #Store Weight for every vertex by distance to target
            for vert in bm.verts:
                distance = (attachedObject.matrix_world.translation - (obj.matrix_world @ vert.co)).length
                weightValue = 1-((distance - minDist) * (1/(maxDist-minDist)))
                vertWeight[vert.index] = weightValue
                verts.append(vert.index)
                
            bpy.ops.object.editmode_toggle()
                
            for vert in verts:
                attachmentInfl.add((vert, vert), vertWeight[vert], 'REPLACE')
              
            attachmentInfl.add(tipVerts, 1, 'REPLACE')
            attachmentInfl.add((0, 0), 0, 'REPLACE')
            attachmentInfl.add((1, 1), 0, 'REPLACE')
            


        #Add LightningMesh----------------------------------------------------------------------------------------------------------------------------------
        mesh = bpy.data.meshes.new("Vert")
        mesh.vertices.add(1)
        object_utils.object_data_add(context, mesh, operator=None)
        bpy.ops.object.mode_set(mode='EDIT')
        bpy.ops.mesh.select_mode(use_extend=False, use_expand=False, type='VERT')

        
        lightningMeshName = setUniqueName("LightningMesh")
        bpy.context.object.name = lightningMeshName
        lightningMesh = bpy.data.objects[lightningMeshName]
        
        me = lightningMesh.data
        bm = bmesh.from_edit_mesh(me)
        
        bpy.ops.mesh.extrude_region_move(MESH_OT_extrude_region={}, TRANSFORM_OT_translate={"value":(0, 0, random.uniform(limit_nZ, limit_pZ)), "orient_type":'GLOBAL', "orient_matrix":((1, 0, 0), (0, 1, 0), (0, 0, 1))})    
        bpy.ops.object.editmode_toggle()
        
        lightningMesh.select_set(False)
        
        lightningMesh.select_set(True)
        bpy.ops.object.editmode_toggle()
        bpy.ops.mesh.select_all(action='DESELECT')
        me = lightningMesh.data
        bm = bmesh.from_edit_mesh(me)

        for i in range(0, branchAmount):
            bm.verts.ensure_lookup_table()
                
            for vert in bm.verts:
                if vert != bm.verts[0]:
                    if vert in vertBranching:
                        vertBranching[vert] = vertBranching[vert]
                    else:
                        vertBranching[vert] = 0
                        
                    if vertBranching[vert] < maxBranchPerPoint:
                        if vert not in availableVerts:
                            availableVerts.append(vert)
                    else:
                        if vert in availableVerts:
                            availableVerts.remove(vert)
                            
                            
            if availableVerts: #If availableVerts has elements in it       
                randomVert = random.choice(availableVerts)
                    
                if vertBranching[randomVert] < maxBranchPerPoint:
                    vertBranching[randomVert] += 1
                else:
                    vertBranching[randomVert] = maxBranchPerPoint
                    
                bpy.ops.mesh.select_all(action='DESELECT')
                randomVert.select = True
                bpy.ops.mesh.extrude_region_move(MESH_OT_extrude_region={}, TRANSFORM_OT_translate={"value":randLocInRange(), "orient_type":'GLOBAL', "orient_matrix":((1, 0, 0), (0, 1, 0), (0, 0, 1))})
            
            
            
        #Adding Vertices to Array for Tip-Thickness Vertx Group
        for vert in bm.verts:
            if vert in vertBranching:
                if vertBranching[vert] == 0:
                    tipVerts.append(vert.index)
            elif vert.index != 0:
                tipVerts.append(vert.index)
                
        bm.verts.ensure_lookup_table()
        for vert in bm.verts:
            if vert != bm.verts[0]:
                middleVerts.append(vert.index)
                if vert.index in tipVerts:
                    middleVerts.remove(vert.index)
            
        bpy.ops.object.editmode_toggle()
        
        
        #Vertex Groups
        displInfluence = lightningMesh.vertex_groups.new(name = 'Displacement Influence No Attachment')
        displInfluence.add((0, 0), 1, 'REPLACE')
        displInfluence.add((1, 1), 0.3, 'REPLACE')
        
        thicknessRoot = lightningMesh.vertex_groups.new(name = 'Root Thickness')
        thicknessRoot.add((0, 0), 1, 'REPLACE')
        thicknessRoot.add((1, 1), 0.5, 'REPLACE')
        
        thicknessMiddle = lightningMesh.vertex_groups.new(name = 'Middle Thickness')
        thicknessMiddle.add(middleVerts, 1, 'REPLACE')
        
        emptyMixGroup = lightningMesh.vertex_groups.new(name = 'Empty Mix Group')
        
        #Add Vertex Groups that are only needed whe attachment is enabled
        if attachedObject is not None and attachEnabled == True:
            vertGroupByDistance(lightningMesh, attachedObject)
        
        
        
        #Add LightningRoot----------------------------------------------------------------------------------------------------------------------------------
        bpy.ops.object.empty_add(type='CIRCLE', align='WORLD', location=lightningMesh.location, scale=(1, 1, 1))
        lightningRootName = setUniqueName("RandomLightning_Root")
        bpy.context.object.name = lightningRootName
        lightningRoot = bpy.data.objects[lightningRootName]
        bpy.ops.transform.rotate(value=1.5708, orient_axis='X', orient_type='GLOBAL', orient_matrix=((1, 0, 0), (0, 1, 0), (0, 0, 1)))
        bpy.ops.transform.resize(value=(0.4, 0.4, 0.4), orient_type='GLOBAL', orient_matrix=((1, 0, 0), (0, 1, 0), (0, 0, 1)))

        bpy.ops.object.empty_add(type='CIRCLE', align='WORLD', location=lightningMesh.location, scale=(1, 1, 1))
        lightningMovName = setUniqueName("LightningMovement")
        bpy.context.object.name = lightningMovName
        lightningMovement = bpy.data.objects[lightningMovName]
        bpy.ops.transform.rotate(value=1.5708, orient_axis='X', orient_type='GLOBAL', orient_matrix=((1, 0, 0), (0, 1, 0), (0, 0, 1)))
        bpy.ops.transform.resize(value=(0.3, 0.3, 0.3), orient_type='GLOBAL', orient_matrix=((1, 0, 0), (0, 1, 0), (0, 0, 1)))

        #Make Root Parent of Mesh and Movement-Object
        lightningMesh.parent = lightningRoot
        lightningMesh.matrix_parent_inverse = lightningRoot.matrix_world.inverted()

        lightningMovement.parent = lightningRoot
        lightningMovement.matrix_parent_inverse = lightningRoot.matrix_world.inverted()
        
        
        #Adding Root LookAt Constraint for Attaching
        if attachedObject is not None and attachEnabled == True:
            attachLookAt = lightningRoot.constraints.new('TRACK_TO')
            attachLookAt.target = attachedObject
            attachLookAt.up_axis = 'UP_Z'
            attachLookAt.track_axis = 'TRACK_NEGATIVE_Y'


        
        
        
        #Adding Effect Modifiers to LighningMesh------------------------------------------------------------------------------------------------------------
        
        #Attached Shrinkwarp if an attachment object is selected 
        if attachedObject is not None and attachEnabled == True:
            #VertexWeightMix for Attachment ajustment if attached to object
            weightMix = lightningMesh.modifiers.new('Attachment Weight Mix', 'VERTEX_WEIGHT_MIX')
            weightMix.vertex_group_a = "Attachment Influence"
            weightMix.vertex_group_b = "Empty Mix Group"
            weightMix.mix_set = 'ALL'
            weightMix.show_expanded = False
 
            attachModifier = lightningMesh.modifiers.new('Attachment Shrinkwarp', 'SHRINKWRAP')
            attachModifier.target = attachedObject
            attachModifier.vertex_group = "Attachment Influence"
            attachModifier.show_expanded = False
        
        
        #Subdividion Modifier 1 (Detail):
        subDModifier_1 = lightningMesh.modifiers.new('Detail Subdivision', 'SUBSURF')
        subDModifier_1.levels = 5
        subDModifier_1.render_levels = 5
        subDModifier_1.show_expanded = False
        
        
        #VertexWeightMix for Displacement ajustment if attached to object
        if attachedObject is not None and attachEnabled == True:
            weightMixDispl = lightningMesh.modifiers.new('Displacement Weight Mix', 'VERTEX_WEIGHT_MIX')
            weightMixDispl.vertex_group_a = "Displacement Influence With Attachment"
            weightMixDispl.vertex_group_b = "Displacement Influence No Attachment"
            weightMixDispl.mix_set = 'ALL'
            weightMixDispl.show_expanded = False


        #Displace X:
        displaceX = lightningMesh.modifiers.new('Displace X', 'DISPLACE')
        
        if attachedObject is not None and attachEnabled == True:
            displaceX.vertex_group = "Displacement Influence With Attachment"
        else:
            displaceX.vertex_group = "Displacement Influence No Attachment"
            
        displaceX.strength = 1
        displaceX.invert_vertex_group = True
        displaceX.texture_coords = 'OBJECT'
        displaceX.texture_coords_object = lightningMovement
        displaceX.direction = 'X'

        xDisplTex = bpy.data.textures.new('DisplacementTextureX', type = 'CLOUDS')
        xDisplTex.noise_scale = 8
        xDisplTex.noise_depth = 4
        xDisplTex.noise_basis = 'BLENDER_ORIGINAL'
        displaceX.texture = xDisplTex

        displaceX.show_expanded = False


        #Displace Y:
        displaceY = lightningMesh.modifiers.new('Displace Y', 'DISPLACE')
        
        if attachedObject is not None and attachEnabled == True:
            displaceY.vertex_group = "Displacement Influence With Attachment"
        else:
            displaceY.vertex_group = "Displacement Influence No Attachment"
        
        displaceY.strength = 1
        displaceY.invert_vertex_group = True
        displaceY.texture_coords = 'OBJECT'
        displaceY.texture_coords_object = lightningMovement
        displaceY.direction = 'Y'

        yDisplTex = bpy.data.textures.new('DisplacementTextureY', type = 'CLOUDS')
        yDisplTex.noise_scale = 8
        yDisplTex.noise_depth = 4
        yDisplTex.noise_basis = 'VORONOI_F1'
        displaceY.texture = yDisplTex

        displaceY.show_expanded = False


        #Displace Z:
        displaceZ = lightningMesh.modifiers.new('Displace Z', 'DISPLACE')
        
        if attachedObject is not None and attachEnabled == True:
            displaceZ.vertex_group = "Displacement Influence With Attachment"
        else:
            displaceZ.vertex_group = "Displacement Influence No Attachment"
        
        displaceZ.strength = 1
        displaceZ.invert_vertex_group = True
        displaceZ.texture_coords = 'OBJECT'
        displaceZ.texture_coords_object = lightningMovement
        displaceZ.direction = 'Z'

        zDisplTex = bpy.data.textures.new('DisplacementTextureZ', type = 'CLOUDS')
        zDisplTex.noise_scale = 8
        zDisplTex.noise_depth = 4
        zDisplTex.noise_basis = 'ORIGINAL_PERLIN'
        displaceZ.texture = zDisplTex

        displaceZ.show_expanded = False


        #Build Start:
        buildStartMod = lightningMesh.modifiers.new('Start', 'BUILD')
        buildStartMod.frame_start = 1
        buildStartMod.frame_duration = 5
        buildStartMod.show_expanded = False


        #Build Start:
        buildStopMod = lightningMesh.modifiers.new('Stop', 'BUILD')
        buildStopMod.frame_start = 50
        buildStopMod.frame_duration = 5
        buildStopMod.use_reverse = True
        buildStopMod.show_expanded = False


        #Subdividion Modifier 2 (Smoothing):
        subDModifier_2 = lightningMesh.modifiers.new('Smoothing Subdivision', 'SUBSURF')
        subDModifier_2.levels = 0
        subDModifier_2.render_levels = 0
        subDModifier_2.show_expanded = False


        #Skin Modifier
        skinModifier = lightningMesh.modifiers.new('Base Thickness', 'SKIN')
        skinModifier.use_x_symmetry = False
        skinModifier.show_expanded = False
        skinModifier.use_smooth_shade = True

        bpy.ops.object.select_all(action="DESELECT")
        lightningMesh.select_set(True)
        bpy.context.view_layer.objects.active = lightningMesh
        bpy.ops.object.editmode_toggle()
        bpy.ops.mesh.select_all(action='SELECT')
        bpy.ops.transform.skin_resize(value=(0.1, 0.1, 0.1), orient_type='GLOBAL', orient_matrix=((1, 0, 0), (0, 1, 0), (0, 0, 1)))
        
        bpy.ops.mesh.select_all(action='DESELECT')

        bm = bmesh.from_edit_mesh(me)
        bm.verts.ensure_lookup_table()

        #Selecting Vertices for thinner Tips
        for vert in tipVerts:
            bm.verts[vert].select = True
            
        bpy.ops.transform.skin_resize(value=(0.01, 0.01, 0.01), orient_type='GLOBAL', orient_matrix=((1, 0, 0), (0, 1, 0), (0, 0, 1)))
        bpy.ops.object.editmode_toggle()
        
        
        #Tickness Middle:
        displaceMiddle = lightningMesh.modifiers.new('Thickness Middle', 'DISPLACE')
        displaceMiddle.vertex_group = "Middle Thickness"
        displaceMiddle.texture_coords = 'LOCAL'
        displaceMiddle.direction = 'NORMAL'
        displaceMiddle.show_expanded = False
        
        
        #Tickness Root:
        displaceRoot = lightningMesh.modifiers.new('Thickness Root', 'DISPLACE')
        displaceRoot.vertex_group = "Root Thickness"
        displaceRoot.texture_coords = 'LOCAL'
        displaceRoot.direction = 'NORMAL'
        displaceRoot.show_expanded = False


        
        #Create and/or Assign Material----------------------------------------------------------------------------------------------------------------------
        mat = bpy.data.materials.get("Lightning Material")
        if mat is None:
            mat = bpy.data.materials.new(name="Lightning Material")

        mat.use_nodes = True
        nodes = mat.node_tree.nodes
        nodes.clear()
        node_emission = nodes.new(type='ShaderNodeEmission')
        node_emission.inputs[0].default_value = (0.121746,0.260608,1,1)     #Color
        node_emission.inputs[1].default_value = 20                          # strength
        node_emission.location = 0,0

        node_output = nodes.new(type='ShaderNodeOutputMaterial')   
        node_output.location = 200,0
        links = mat.node_tree.links
        link = links.new(node_emission.outputs[0], node_output.inputs[0])

        node_ramp = nodes.new(type='ShaderNodeValToRGB')   
        node_ramp.location = -300,0
        link = links.new(node_ramp.outputs[0], node_emission.inputs[0])
        node_ramp.color_ramp.elements[0].color = (0.127438, 0.386429, 1, 1)
        node_ramp.color_ramp.elements[1].color = (0.391573, 0.061246, 1, 1)

        node_layerWeight = nodes.new(type='ShaderNodeLayerWeight')  
        node_layerWeight.location = -500,0
        link = links.new(node_layerWeight.outputs[1], node_ramp.inputs[0])
        node_layerWeight.inputs[0].default_value = 0.8

        lightningMesh.data.materials.append(mat)
        
        
        
        #Create custom properties with drivers--------------------------------------------------------------------------------------------------------------

        #Detail Subdivision
        lightningMesh["Lightning Detail"] = 5
        lnDetail_Prop = rna_idprop_ui_create(lightningMesh, 'Lightning Detail', default = 5, min = 0, max = 6, soft_min = 0, soft_max = 6, description = 'The resolution used for displacement')
        add_driver(lightningMesh, 'modifiers["Detail Subdivision"].levels', '["Lightning Detail"]', 'Lightning_Detail', 'Lightning_Detail')
        add_driver(lightningMesh, 'modifiers["Detail Subdivision"].render_levels', '["Lightning Detail"]', 'Lightning_Detail', 'Lightning_Detail')
        
        #Smoothing Subdivision
        lightningMesh["Lightning Smoothing"] = 0
        lnSmooth_Prop = rna_idprop_ui_create(lightningMesh, 'Lightning Smoothing', default = 0, min = 0, max = 6, soft_min = 0, soft_max = 6, description = 'Smoothing of the lightning')
        add_driver(lightningMesh, 'modifiers["Smoothing Subdivision"].levels', '["Lightning Smoothing"]', 'Lightning_Smoothing', 'Lightning_Smoothing')
        add_driver(lightningMesh, 'modifiers["Smoothing Subdivision"].render_levels', '["Lightning Smoothing"]', 'Lightning_Smoothing', 'Lightning_Smoothing')
        
        #Start Time
        lightningMesh["Start Time"] = 0
        lnStart_Prop = rna_idprop_ui_create(lightningMesh, 'Start Time', default = current_frame - 5, min = -999999999, max = 999999999, description = 'The time when the lightning starts to develop')
        add_driver(lightningMesh, 'modifiers["Start"].frame_start', '["Start Time"]', 'Start_Time', 'Start_Time')
        
        #Stop Time
        lightningMesh["Stop Time"] = 0
        lnStop_Prop = rna_idprop_ui_create(lightningMesh, 'Stop Time', default = current_frame + 50, min = -999999999, max = 999999999, description = 'The time when the lightning starts to disappear')
        add_driver(lightningMesh, 'modifiers["Stop"].frame_start', '["Stop Time"]', 'Stop_Time', 'Stop_Time')
        
        #Start Duration
        lightningMesh["Start Duration"] = 5
        lnStartDur_Prop = rna_idprop_ui_create(lightningMesh, 'Start Duration', default = 5, min = 0, max = 999999999, soft_min = 0, description = 'The time it takes for the lightning to develop')
        add_driver(lightningMesh, 'modifiers["Start"].frame_duration', '["Start Duration"]', 'Start_Duration', 'Start_Duration')
        
        #Stop Duration
        lightningMesh["Stop Duration"] = 5
        lnStopDur_Prop = rna_idprop_ui_create(lightningMesh, 'Stop Duration', default = 5, min = 0, max = 999999999, soft_min = 0, description = 'The time it takes for the lightning to disappear')
        add_driver(lightningMesh, 'modifiers["Stop"].frame_duration', '["Stop Duration"]', 'Stop_Duration', 'Stop_Duration')
        
        #Distortion Strength
        lightningMesh["Distortion Strength"] = 4.0
        lnDistStr_Prop = rna_idprop_ui_create(lightningMesh, 'Distortion Strength', default = 4.0, min = 0.0, max = 999999999.0, soft_min = 0.0, description = 'Strength of the distortion of the lightning')
        add_driver(lightningMesh, 'modifiers["Displace X"].strength', '["Distortion Strength"]', 'Distortion_Strength', 'Distortion_Strength')
        add_driver(lightningMesh, 'modifiers["Displace Y"].strength', '["Distortion Strength"]', 'Distortion_Strength', 'Distortion_Strength')
        add_driver(lightningMesh, 'modifiers["Displace Z"].strength', '["Distortion Strength"]', 'Distortion_Strength', 'Distortion_Strength')
        
        #Thickness Root
        lightningMesh["Thickness Root"] = 0.1
        lnThickRoot_Prop = rna_idprop_ui_create(lightningMesh, 'Thickness Root', default = 0.1, min = 0.0, max = 1.0, soft_min = 0, soft_max = 1.0, description = 'The thickness of the lightning at the root')
        add_driver(lightningMesh, 'modifiers["Thickness Root"].strength', '["Thickness Root"]', 'Thickness_Root', 'Thickness_Root /2')
        
        #Thickness Middle
        lightningMesh["Thickness Middle"] = 0.05
        lnThickMid_Prop = rna_idprop_ui_create(lightningMesh, 'Thickness Middle', default = 0.05, min = 0.0, max = 1.0, soft_min = 0.0, soft_max = 1.0, description = 'The thickness of the lightning in the middle')
        add_driver(lightningMesh, 'modifiers["Thickness Middle"].strength', '["Thickness Middle"]', 'Thickness_Middle', '(Thickness_Middle /2)-0.03')

        #Root Look-At
        if attachedObject is not None and attachEnabled == True:
            lightningMesh["Attachment Strength"] = 1.0
            lookAt_Influence_Prop = rna_idprop_ui_create(lightningMesh, 'Attachment Strength', default = 1.0, min = 0.0, max = 1.0, soft_min = 0.0, soft_max = 1.0, description = 'How much the root object points at the target object, if the lightning is attached to an object')
            add_driverAO(lightningRoot, lightningMesh, 'constraints["Track To"].influence', '["Attachment Strength"]', 'Attachment_Strength', 'Attachment_Strength')        
            add_driver(lightningMesh, 'modifiers["Attachment Weight Mix"].mask_constant', '["Attachment Strength"]', 'Attachment_Strength', '(1 - Attachment_Strength)') 
            add_driver(lightningMesh, 'modifiers["Displacement Weight Mix"].mask_constant', '["Attachment Strength"]', 'Attachment_Strength', '(1 - Attachment_Strength)')
        
        scene.tool_settings.transform_pivot_point = savePPoint

        return {'FINISHED'} 


    
def register() :
    bpy.utils.register_class(AddRandomLightning)
 
def unregister() :
    bpy.utils.unregister_class(AddRandomLightning)