import bpy
import bmesh
import sys
from rna_prop_ui import rna_idprop_ui_create
from bpy_extras import object_utils

class AddSimpleLightning(bpy.types.Operator):
    bl_label = "New Simple Lightning"
    bl_idname = 'lightning.simple'
    bl_options = {"REGISTER", "UNDO"}
    
    @classmethod
    def description(cls, context, properties):
        return "Adds a new 'Simple Lightning' at the position of the 3D cursor"
    
    def execute(self, context):        

        scene = bpy.context.scene
        savePPoint = scene.tool_settings.transform_pivot_point
        scene.tool_settings.transform_pivot_point = 'MEDIAN_POINT'
        
        cursorLoc = bpy.context.scene.cursor.location
        current_frame = bpy.data.scenes[0].frame_current


        #Check for already existing object with name--------------------------------------------------------------------------------------------------------
        def setUniqueName(newName):
            if bpy.data.objects.get(newName) is not None:
                checkName = newName
                counter = 1
                while bpy.data.objects.get(checkName) is not None:
                    counter += 1
                    checkName = newName + "_" + str(counter)
                return checkName
            else:
                return newName
            
            
            
        #Create Driver--------------------------------------------------------------------------------------------------------------------------------------
        def add_driver(source, prop, dataPath, name = '',func = ''):
            d = source.driver_add( prop ).driver
            v = d.variables.new()
            v.name                 = name
            v.targets[0].id        = source
            v.targets[0].data_path = dataPath
            d.expression = func



        #Add LightningMesh----------------------------------------------------------------------------------------------------------------------------------
        
        mesh = bpy.data.meshes.new("Vert")
        mesh.vertices.add(1)
        object_utils.object_data_add(context, mesh, operator=None)
        bpy.ops.object.mode_set(mode='EDIT')
        bpy.ops.mesh.select_mode(use_extend=False, use_expand=False, type='VERT')

        
        lightningMeshName = setUniqueName("LightningMesh")
        bpy.context.object.name = lightningMeshName
        lightningMesh = bpy.data.objects[lightningMeshName]
        bpy.ops.mesh.extrude_region_move(MESH_OT_extrude_region={}, TRANSFORM_OT_translate={"value":(-1, -0, -0), "orient_type":'GLOBAL', "orient_matrix":((1, 0, 0), (0, 1, 0), (0, 0, 1))})
        bpy.ops.mesh.extrude_region_move(MESH_OT_extrude_region={}, TRANSFORM_OT_translate={"value":(-1, -0, -0), "orient_type":'GLOBAL', "orient_matrix":((1, 0, 0), (0, 1, 0), (0, 0, 1))})
        bpy.ops.mesh.extrude_region_move(MESH_OT_extrude_region={}, TRANSFORM_OT_translate={"value":(-1, -0, -0), "orient_type":'GLOBAL', "orient_matrix":((1, 0, 0), (0, 1, 0), (0, 0, 1))})
        bpy.ops.object.editmode_toggle()
        bpy.ops.object.origin_set(type='ORIGIN_GEOMETRY', center='MEDIAN')
        bpy.context.object.lock_location[0] = True
        bpy.context.object.lock_location[1] = True
        bpy.context.object.lock_location[2] = True
        bpy.context.object.lock_rotation[0] = True
        bpy.context.object.lock_rotation[1] = True
        bpy.context.object.lock_rotation[2] = True

        #Adding Vertex Groups
        influenceGroup = lightningMesh.vertex_groups.new(name = 'Displacement Influence')
        influenceGroup.add((1, 2), 1, 'REPLACE')
        
        influenceRight = lightningMesh.vertex_groups.new(name = 'Displacement Influence Right')
        influenceRight.add((3, 3), 1, 'REPLACE')
        influenceRight.add((1, 2), 0, 'REPLACE')
        
        influenceLeft = lightningMesh.vertex_groups.new(name = 'Displacement Influence Left')
        influenceLeft.add((0, 0), 1, 'REPLACE')
        influenceLeft.add((1, 2), 0, 'REPLACE')
        



        #Add LightningRoot----------------------------------------------------------------------------------------------------------------------------------
        bpy.ops.object.empty_add(type='CIRCLE', align='WORLD', location=lightningMesh.location, scale=(1, 1, 1))
        lightningRootName = setUniqueName("SimpleLightning_Root")
        bpy.context.object.name = lightningRootName
        lightningRoot = bpy.data.objects[lightningRootName]
        bpy.ops.transform.rotate(value=1.5708, orient_axis='Z', orient_type='GLOBAL', orient_matrix=((1, 0, 0), (0, 1, 0), (0, 0, 1)))
        bpy.ops.transform.resize(value=(0.3, 0.3, 0.3), orient_type='GLOBAL', orient_matrix=((1, 0, 0), (0, 1, 0), (0, 0, 1)))

        bpy.ops.object.empty_add(type='CIRCLE', align='WORLD', location=lightningMesh.location, scale=(1, 1, 1))
        lightningMovName = setUniqueName("LightningMovement")
        bpy.context.object.name = lightningMovName
        lightningMovement = bpy.data.objects[lightningMovName]
        bpy.ops.transform.rotate(value=1.5708, orient_axis='Z', orient_type='GLOBAL', orient_matrix=((1, 0, 0), (0, 1, 0), (0, 0, 1)))
        bpy.ops.transform.resize(value=(0.25, 0.25, 0.25), orient_type='GLOBAL', orient_matrix=((1, 0, 0), (0, 1, 0), (0, 0, 1)))

        #Make Root Parent of Mesh
        lightningMesh.parent = lightningRoot
        lightningMesh.matrix_parent_inverse = lightningRoot.matrix_world.inverted()

        lightningMovement.parent = lightningRoot
        lightningMovement.matrix_parent_inverse = lightningRoot.matrix_world.inverted()



        #Add Lightning Handles------------------------------------------------------------------------------------------------------------------------------
        #Add Hook 1
        bpy.ops.object.empty_add(type='SPHERE', align='WORLD', location=cursorLoc, scale=(1, 1, 1))
        bpy.ops.transform.resize(value=(0.1, 0.1, 0.1), orient_type='GLOBAL', orient_matrix=((1, 0, 0), (0, 1, 0), (0, 0, 1)))
        controlName1 = setUniqueName("LightningCTRL_1")
        bpy.context.object.name = controlName1
        lightningCTRL_1 = bpy.data.objects[controlName1]

        bpy.ops.mesh.primitive_circle_add(vertices=3, enter_editmode=False, align='WORLD', location=cursorLoc, scale=(1, 1, 1))
        lightningSWarpName = setUniqueName("LightningSWarp_1")
        bpy.context.object.name = lightningSWarpName
        lightningSWarp_1 = bpy.data.objects[lightningSWarpName]
        bpy.ops.transform.rotate(value=-1.5708, orient_axis='Z', orient_type='GLOBAL', orient_matrix=((1, 0, 0), (0, 1, 0), (0, 0, 1)))
        bpy.ops.transform.rotate(value=-1.5708, orient_axis='Y', orient_type='GLOBAL', orient_matrix=((1, 0, 0), (0, 1, 0), (0, 0, 1)))
        bpy.ops.transform.resize(value=(0.02, 0.02, 0.02))
        bpy.ops.object.modifier_add(type='SHRINKWRAP')
        bpy.context.object.lock_location[0] = True
        bpy.context.object.lock_location[1] = True
        bpy.context.object.lock_location[2] = True
        bpy.context.object.lock_rotation[0] = True
        bpy.context.object.lock_rotation[1] = True
        bpy.context.object.lock_rotation[2] = True

        bpy.ops.object.empty_add(type='CUBE', align='WORLD', location=cursorLoc, scale=(1, 1, 1))
        bpy.ops.transform.resize(value=(0.02, 0.02, 0.02), orient_type='GLOBAL', orient_matrix=((1, 0, 0), (0, 1, 0), (0, 0, 1)))
        bpy.context.object.lock_location[0] = True
        bpy.context.object.lock_location[1] = True
        bpy.context.object.lock_location[2] = True
        bpy.context.object.lock_rotation[0] = True
        bpy.context.object.lock_rotation[1] = True
        bpy.context.object.lock_rotation[2] = True
        hookName1 = setUniqueName("LightningHook_1")
        bpy.context.object.name = hookName1
        lightningHook_1 = bpy.data.objects[hookName1]

        modifier1 = lightningMesh.modifiers.new("Hook1", 'HOOK')
        modifier1.object = lightningHook_1
        modifier1.vertex_indices_set([0])
        modifier1.show_expanded = False

        lightningHook_1.parent = lightningSWarp_1
        lightningHook_1.parent_type = 'VERTEX_3'
        lightningHook_1.parent_vertices[0] = 1
        lightningHook_1.parent_vertices[1] = 2
        lightningHook_1.parent_vertices[2] = 0
        lightningHook_1.matrix_parent_inverse = lightningSWarp_1.matrix_world.inverted()

        lightningSWarp_1.parent = lightningCTRL_1
        lightningSWarp_1.matrix_parent_inverse = lightningCTRL_1.matrix_world.inverted()

        lightningCTRL_1.parent = lightningRoot
        lightningCTRL_1.matrix_parent_inverse = lightningRoot.matrix_world.inverted()


        #Add Hook 2
        bpy.ops.object.empty_add(type='SPHERE', align='WORLD', location=cursorLoc, scale=(1, 1, 1))
        bpy.ops.transform.resize(value=(0.1, 0.1, 0.1), orient_type='GLOBAL', orient_matrix=((1, 0, 0), (0, 1, 0), (0, 0, 1)))
        controlName2 = setUniqueName("LightningCTRL_2")
        bpy.context.object.name = controlName2
        lightningCTRL_2 = bpy.data.objects[controlName2]
        bpy.ops.transform.translate(value=(-1, -0, -0), orient_type='GLOBAL', orient_matrix=((1, 0, 0), (0, 1, 0), (0, 0, 1)))

        modifier2 = lightningMesh.modifiers.new("Hook2", 'HOOK')
        modifier2.object = lightningCTRL_2
        modifier2.vertex_indices_set([1])
        modifier2.show_expanded = False

        lightningCTRL_2.parent = lightningHook_1
        lightningCTRL_2.matrix_parent_inverse = lightningHook_1.matrix_world.inverted()


        #Arrow 1
        bpy.ops.object.empty_add(type='SINGLE_ARROW', align='WORLD', location=lightningCTRL_1.location, scale=(1, 1, 1))
        bpy.ops.transform.resize(value=(0.5, 0.5, 0.5), orient_type='GLOBAL', orient_matrix=((1, 0, 0), (0, 1, 0), (0, 0, 1)))
        bpy.ops.transform.rotate(value=1.5708, orient_axis='Y', orient_type='GLOBAL', orient_matrix=((1, 0, 0), (0, 1, 0), (0, 0, 1)))
        bpy.context.object.lock_location[0] = True
        bpy.context.object.lock_location[1] = True
        bpy.context.object.lock_location[2] = True
        bpy.context.object.lock_rotation[0] = True
        bpy.context.object.lock_rotation[1] = True
        bpy.context.object.lock_rotation[2] = True
        arrowName1 = setUniqueName("LightningCTRL_1_Arrow")
        bpy.context.object.name = arrowName1
        lightningCTRL_1_Arrow = bpy.data.objects[arrowName1]

        lightningCTRL_1_Arrow.parent = lightningCTRL_1
        lightningCTRL_1_Arrow.matrix_parent_inverse = lightningCTRL_1.matrix_world.inverted()

        bpy.ops.object.constraint_add(type='TRACK_TO')
        bpy.context.object.constraints["Track To"].target = lightningCTRL_2


        #Add Hook 4
        bpy.ops.object.empty_add(type='SPHERE', align='WORLD', location=cursorLoc, scale=(1, 1, 1))
        bpy.ops.transform.resize(value=(0.1, 0.1, 0.1), orient_type='GLOBAL', orient_matrix=((1, 0, 0), (0, 1, 0), (0, 0, 1)))
        controlName4 = setUniqueName("LightningCTRL_4")
        bpy.context.object.name = controlName4
        lightningCTRL_4 = bpy.data.objects[controlName4]
        bpy.ops.transform.translate(value=(-3, -0, -0), orient_type='GLOBAL', orient_matrix=((1, 0, 0), (0, 1, 0), (0, 0, 1)))

        bpy.ops.mesh.primitive_circle_add(vertices=3, enter_editmode=False, align='WORLD', location=cursorLoc, scale=(1, 1, 1))
        lightningSWarpName2 = setUniqueName("LightningSWarp_2")
        bpy.context.object.name = lightningSWarpName2
        lightningSWarp_2 = bpy.data.objects[lightningSWarpName2]
        bpy.ops.transform.rotate(value=-1.5708, orient_axis='Z', orient_type='GLOBAL', orient_matrix=((1, 0, 0), (0, 1, 0), (0, 0, 1)))
        bpy.ops.transform.rotate(value=-1.5708, orient_axis='Y', orient_type='GLOBAL', orient_matrix=((1, 0, 0), (0, 1, 0), (0, 0, 1)))
        bpy.ops.transform.resize(value=(0.02, 0.02, 0.02))
        bpy.ops.transform.translate(value=(-3, -0, -0), orient_type='GLOBAL', orient_matrix=((1, 0, 0), (0, 1, 0), (0, 0, 1)))
        bpy.ops.object.modifier_add(type='SHRINKWRAP')
        bpy.context.object.lock_location[0] = True
        bpy.context.object.lock_location[1] = True
        bpy.context.object.lock_location[2] = True
        bpy.context.object.lock_rotation[0] = True
        bpy.context.object.lock_rotation[1] = True
        bpy.context.object.lock_rotation[2] = True


        bpy.ops.object.empty_add(type='CUBE', align='WORLD', location=cursorLoc, scale=(1, 1, 1))
        bpy.ops.transform.resize(value=(0.02, 0.02, 0.02), orient_type='GLOBAL', orient_matrix=((1, 0, 0), (0, 1, 0), (0, 0, 1)))
        hookName2 = setUniqueName("LightningHook_2")
        bpy.context.object.name = hookName2
        lightningHook_2 = bpy.data.objects[hookName2]
        bpy.ops.transform.translate(value=(-3, -0, -0), orient_type='GLOBAL', orient_matrix=((1, 0, 0), (0, 1, 0), (0, 0, 1)))
        bpy.context.object.lock_location[0] = True
        bpy.context.object.lock_location[1] = True
        bpy.context.object.lock_location[2] = True
        bpy.context.object.lock_rotation[0] = True
        bpy.context.object.lock_rotation[1] = True
        bpy.context.object.lock_rotation[2] = True

        modifier4 = lightningMesh.modifiers.new("Hook3", 'HOOK')
        modifier4.object = lightningHook_2
        modifier4.vertex_indices_set([3])
        modifier4.show_expanded = False

        lightningHook_2.parent = lightningSWarp_2
        lightningHook_2.parent_type = 'VERTEX_3'
        lightningHook_2.parent_vertices[0] = 1
        lightningHook_2.parent_vertices[1] = 2
        lightningHook_2.parent_vertices[2] = 0
        lightningHook_2.matrix_parent_inverse = lightningSWarp_2.matrix_world.inverted()

        lightningSWarp_2.parent = lightningCTRL_4
        lightningSWarp_2.matrix_parent_inverse = lightningCTRL_4.matrix_world.inverted()

        lightningCTRL_4.parent = lightningRoot
        lightningCTRL_4.matrix_parent_inverse = lightningRoot.matrix_world.inverted()


        #Add Hook 3
        bpy.ops.object.empty_add(type='SPHERE', align='WORLD', location=cursorLoc, scale=(1, 1, 1))
        bpy.ops.transform.resize(value=(0.1, 0.1, 0.1), orient_type='GLOBAL', orient_matrix=((1, 0, 0), (0, 1, 0), (0, 0, 1)))
        controlName3 = setUniqueName("LightningCTRL_3")
        bpy.context.object.name = controlName3
        lightningCTRL_3 = bpy.data.objects[controlName3]
        bpy.ops.transform.translate(value=(-2, -0, -0), orient_type='GLOBAL', orient_matrix=((1, 0, 0), (0, 1, 0), (0, 0, 1)))

        modifier3 = lightningMesh.modifiers.new("Hook4", 'HOOK')
        modifier3.object = lightningCTRL_3
        modifier3.vertex_indices_set([2])
        modifier3.show_expanded = False

        lightningCTRL_3.parent = lightningHook_2
        lightningCTRL_3.matrix_parent_inverse = lightningHook_2.matrix_world.inverted()


        #Arrow 2
        bpy.ops.object.empty_add(type='SINGLE_ARROW', align='WORLD', location=lightningCTRL_4.location, scale=(1, 1, 1))
        bpy.ops.transform.resize(value=(0.5, 0.5, 0.5), orient_type='GLOBAL', orient_matrix=((1, 0, 0), (0, 1, 0), (0, 0, 1)))
        bpy.ops.transform.rotate(value=1.5708, orient_axis='Y', orient_type='GLOBAL', orient_matrix=((1, 0, 0), (0, 1, 0), (0, 0, 1)))
        bpy.context.object.lock_location[0] = True
        bpy.context.object.lock_location[1] = True
        bpy.context.object.lock_location[2] = True
        bpy.context.object.lock_rotation[0] = True
        bpy.context.object.lock_rotation[1] = True
        bpy.context.object.lock_rotation[2] = True
        arrowName2 = setUniqueName("LightningCTRL_4_Arrow")
        bpy.context.object.name = arrowName2
        lightningCTRL_4_Arrow = bpy.data.objects[arrowName2]

        lightningCTRL_4_Arrow.parent = lightningCTRL_4
        lightningCTRL_4_Arrow.matrix_parent_inverse = lightningCTRL_4.matrix_world.inverted()

        bpy.ops.object.constraint_add(type='TRACK_TO')
        bpy.context.object.constraints["Track To"].target = lightningCTRL_3


        #Move All to Cursor   
        lightningRoot.matrix_world.translation = scene.cursor.location



        #Adding Effect Modifiers to LighningMesh------------------------------------------------------------------------------------------------------------

        #Subdividion Modifier 1 (Detail):
        subDModifier_1 = lightningMesh.modifiers.new('Detail Subdivision', 'SUBSURF')
        subDModifier_1.levels = 6
        subDModifier_1.render_levels = 6
        subDModifier_1.show_expanded = False


        #Displace X:
        displaceX = lightningMesh.modifiers.new('Displace X', 'DISPLACE')
        displaceX.vertex_group = "Displacement Influence"
        displaceX.texture_coords = 'OBJECT'
        displaceX.texture_coords_object = lightningMovement
        displaceX.direction = 'X'

        xDisplTex = bpy.data.textures.new('DisplacementTextureX', type = 'CLOUDS')
        xDisplTex.noise_scale = 2
        xDisplTex.noise_basis = 'BLENDER_ORIGINAL'
        displaceX.texture = xDisplTex

        displaceX.show_expanded = False


        #Displace Y:
        displaceY = lightningMesh.modifiers.new('Displace Y', 'DISPLACE')
        displaceY.vertex_group = "Displacement Influence"
        displaceY.texture_coords = 'OBJECT'
        displaceY.texture_coords_object = lightningMovement
        displaceY.direction = 'Y'

        yDisplTex = bpy.data.textures.new('DisplacementTextureY', type = 'CLOUDS')
        yDisplTex.noise_scale = 2
        yDisplTex.noise_basis = 'VORONOI_F1'
        displaceY.texture = yDisplTex

        displaceY.show_expanded = False


        #Displace Z:
        displaceZ = lightningMesh.modifiers.new('Displace Z', 'DISPLACE')
        displaceZ.vertex_group = "Displacement Influence"
        displaceZ.texture_coords = 'OBJECT'
        displaceZ.texture_coords_object = lightningMovement
        displaceZ.direction = 'Z'

        zDisplTex = bpy.data.textures.new('DisplacementTextureZ', type = 'CLOUDS')
        zDisplTex.noise_scale = 2
        zDisplTex.noise_basis = 'ORIGINAL_PERLIN'
        displaceZ.texture = zDisplTex

        displaceZ.show_expanded = False


        #Build Start:
        buildStartMod = lightningMesh.modifiers.new('Start', 'BUILD')
        buildStartMod.frame_start = 1
        buildStartMod.frame_duration = 5
        buildStartMod.show_expanded = False


        #Build Start:
        buildStopMod = lightningMesh.modifiers.new('Stop', 'BUILD')
        buildStopMod.frame_start = 50
        buildStopMod.frame_duration = 5
        buildStopMod.use_reverse = True
        buildStopMod.show_expanded = False


        #Subdividion Modifier 2 (Smoothing):
        subDModifier_2 = lightningMesh.modifiers.new('Smoothing Subdivision', 'SUBSURF')
        subDModifier_2.levels = 1
        subDModifier_2.render_levels = 1
        subDModifier_2.show_expanded = False


        #Skin Modifier
        skinModifier = lightningMesh.modifiers.new('Base Thickness', 'SKIN')
        skinModifier.use_x_symmetry = False
        skinModifier.show_expanded = False
        skinModifier.use_smooth_shade = True

        bpy.ops.object.select_all(action="DESELECT")
        lightningMesh.select_set(True)
        bpy.context.view_layer.objects.active = lightningMesh
        bpy.ops.object.editmode_toggle()
        bpy.ops.mesh.select_all(action='SELECT')
        bpy.ops.transform.skin_resize(value=(0.01, 0.01, 0.01), orient_type='GLOBAL', orient_matrix=((1, 0, 0), (0, 1, 0), (0, 0, 1)))

        bpy.ops.mesh.select_all(action='DESELECT')
        bpy.ops.object.editmode_toggle()
        
        
        #Tickness Middle:
        displaceMiddle = lightningMesh.modifiers.new('Thickness Middle', 'DISPLACE')
        displaceMiddle.vertex_group = "Displacement Influence"
        displaceMiddle.texture_coords = 'LOCAL'
        displaceMiddle.direction = 'NORMAL'
        displaceMiddle.show_expanded = False
        
        
        #Tickness Right:
        displaceRight = lightningMesh.modifiers.new('Thickness Right', 'DISPLACE')
        displaceRight.vertex_group = "Displacement Influence Right"
        displaceRight.texture_coords = 'LOCAL'
        displaceRight.direction = 'NORMAL'
        displaceRight.show_expanded = False
        
        
        #Tickness Left:
        displaceLeft = lightningMesh.modifiers.new('Thickness Left', 'DISPLACE')
        displaceLeft.vertex_group = "Displacement Influence Left"
        displaceLeft.texture_coords = 'LOCAL'
        displaceLeft.direction = 'NORMAL'
        displaceLeft.show_expanded = False



        #Create and/or Assign Material----------------------------------------------------------------------------------------------------------------------
        mat = bpy.data.materials.get("Lightning Material")
        if mat is None:
            mat = bpy.data.materials.new(name="Lightning Material")

        mat.use_nodes = True
        nodes = mat.node_tree.nodes
        nodes.clear()
        node_emission = nodes.new(type='ShaderNodeEmission')
        node_emission.inputs[0].default_value = (0.121746,0.260608,1,1)     #Color
        node_emission.inputs[1].default_value = 20                          # strength
        node_emission.location = 0,0

        node_output = nodes.new(type='ShaderNodeOutputMaterial')   
        node_output.location = 200,0
        links = mat.node_tree.links
        link = links.new(node_emission.outputs[0], node_output.inputs[0])

        node_ramp = nodes.new(type='ShaderNodeValToRGB')   
        node_ramp.location = -300,0
        link = links.new(node_ramp.outputs[0], node_emission.inputs[0])
        node_ramp.color_ramp.elements[0].color = (0.127438, 0.386429, 1, 1)
        node_ramp.color_ramp.elements[1].color = (0.391573, 0.061246, 1, 1)

        node_layerWeight = nodes.new(type='ShaderNodeLayerWeight')  
        node_layerWeight.location = -500,0
        link = links.new(node_layerWeight.outputs[1], node_ramp.inputs[0])
        node_layerWeight.inputs[0].default_value = 0.8

        lightningMesh.data.materials.append(mat)



        #Create custom properties with drivers--------------------------------------------------------------------------------------------------------------

        #Detail Subdivision
        lightningMesh["Lightning Detail"] = 5
        lnDetail_Prop = rna_idprop_ui_create(lightningMesh, 'Lightning Detail', default = 5, min = 0, max = 6, soft_min = 0, soft_max = 6, description = 'The resolution used for displacement')
        add_driver(lightningMesh, 'modifiers["Detail Subdivision"].levels', '["Lightning Detail"]', 'Lightning_Detail', 'Lightning_Detail')
        add_driver(lightningMesh, 'modifiers["Detail Subdivision"].render_levels', '["Lightning Detail"]', 'Lightning_Detail', 'Lightning_Detail')
        
        #Smoothing Subdivision
        lightningMesh["Lightning Smoothing"] = 0
        lnSmooth_Prop = rna_idprop_ui_create(lightningMesh, 'Lightning Smoothing', default = 0, min = 0, max = 6, soft_min = 0, soft_max = 6, description = 'Smoothing of the lightning')
        add_driver(lightningMesh, 'modifiers["Smoothing Subdivision"].levels', '["Lightning Smoothing"]', 'Lightning_Smoothing', 'Lightning_Smoothing')
        add_driver(lightningMesh, 'modifiers["Smoothing Subdivision"].render_levels', '["Lightning Smoothing"]', 'Lightning_Smoothing', 'Lightning_Smoothing')
        
        #Start Time
        lightningMesh["Start Time"] = 0
        lnStart_Prop = rna_idprop_ui_create(lightningMesh, 'Start Time', default = current_frame - 5, min = -999999999, max = 999999999, description = 'The time when the lightning starts to develop')
        add_driver(lightningMesh, 'modifiers["Start"].frame_start', '["Start Time"]', 'Start_Time', 'Start_Time')
        
        #Stop Time
        lightningMesh["Stop Time"] = 0
        lnStop_Prop = rna_idprop_ui_create(lightningMesh, 'Stop Time', default = current_frame + 50, min = -999999999, max = 999999999, description = 'The time when the lightning starts to disappear')
        add_driver(lightningMesh, 'modifiers["Stop"].frame_start', '["Stop Time"]', 'Stop_Time', 'Stop_Time')
        
        #Start Duration
        lightningMesh["Start Duration"] = 5
        lnStartDur_Prop = rna_idprop_ui_create(lightningMesh, 'Start Duration', default = 5, min = 0, max = 999999999, soft_min = 0, description = 'The time it takes for the lightning to develop')
        add_driver(lightningMesh, 'modifiers["Start"].frame_duration', '["Start Duration"]', 'Start_Duration', 'Start_Duration')
        
        #Stop Duration
        lightningMesh["Stop Duration"] = 5
        lnStopDur_Prop = rna_idprop_ui_create(lightningMesh, 'Stop Duration', default = 5, min = 0, max = 999999999, soft_min = 0, description = 'The time it takes for the lightning to disappear')
        add_driver(lightningMesh, 'modifiers["Stop"].frame_duration', '["Stop Duration"]', 'Stop_Duration', 'Stop_Duration')
        
        #Distortion Strength
        lightningMesh["Distortion Strength"] = 1.0
        lnDistStr_Prop = rna_idprop_ui_create(lightningMesh, 'Distortion Strength', default = 1.0, min = 0.0, max = 999999999.0, soft_min = 0.0, description = 'Strength of the distortion of the lightning')
        add_driver(lightningMesh, 'modifiers["Displace X"].strength', '["Distortion Strength"]', 'Distortion_Strength', 'Distortion_Strength')
        add_driver(lightningMesh, 'modifiers["Displace Y"].strength', '["Distortion Strength"]', 'Distortion_Strength', 'Distortion_Strength')
        add_driver(lightningMesh, 'modifiers["Displace Z"].strength', '["Distortion Strength"]', 'Distortion_Strength', 'Distortion_Strength')
               
        #Thickness Middle
        lightningMesh["Thickness Middle"] = 0.15
        lnThickMid_Prop = rna_idprop_ui_create(lightningMesh, 'Thickness Middle', default = 0.15, min = 0.0, max = 1.0, soft_min = 0.0, soft_max = 1.0, description = 'The thickness of the lightning in the middle')
        add_driver(lightningMesh, 'modifiers["Thickness Middle"].strength', '["Thickness Middle"]', 'Thickness_Middle', '(Thickness_Middle /2)-0.03')

        
        #Thickness Right
        lightningMesh["Thickness Right Tip"] = 0.01
        lnThickRight_Prop = rna_idprop_ui_create(lightningMesh, 'Thickness Right Tip', default = 0.01, min = 0.0, max = 1.0, soft_min = 0.0, soft_max = 1.0, description = 'The thickness of the right tip of the lightning')
        add_driver(lightningMesh, 'modifiers["Thickness Right"].strength', '["Thickness Right Tip"]', 'Thickness_Right_Tip', 'Thickness_Right_Tip /2')
        
        #Thickness Left
        lightningMesh["Thickness Left Tip"] = 0.01
        lnThickLeft_Prop = rna_idprop_ui_create(lightningMesh, 'Thickness Left Tip', default = 0.01, min = 0.0, max = 1.0, soft_min = 0.0, soft_max = 1.0, description = 'The thickness of the right tip of the lightning')
        add_driver(lightningMesh, 'modifiers["Thickness Left"].strength', '["Thickness Left Tip"]', 'Thickness_Left_Tip', 'Thickness_Left_Tip /2')
        
        scene.tool_settings.transform_pivot_point = savePPoint   
        
        return {'FINISHED'} 
        
        
def register() :
    bpy.utils.register_class(AddSimpleLightning)
 
def unregister() :
    bpy.utils.unregister_class(AddSimpleLightning)