import bpy
import os
import bpy.utils.previews
from bpy.props import (StringProperty,
                       BoolProperty,
                       IntProperty,
                       FloatProperty,
                       EnumProperty,
                       PointerProperty,
                       FloatVectorProperty,
                       )

from bpy.types import (Panel,
                       PropertyGroup
                       )

#Load all png Files from file-path where this py-file is
preview_collections = {}
dir = os.path.dirname(os.path.abspath(__file__))

pcoll = bpy.utils.previews.new()

for entry in os.scandir(dir):
    if entry.name.endswith(".png"):
        name = os.path.splitext(entry.name)[0]
        pcoll.load(name.upper(), entry.path, "IMAGE")


#Properties ----------------------------------------------------------------------------------------------------------------------------------------------------------------------
class ElectrifyProperties(PropertyGroup):

    randL_BranchAmount: IntProperty(
        name = "Branch Amount",
        description="The amount of branches the lightning has.\nNote: High values can impact performance!",
        default = 20,
        min = 0,
        max = 100
        )
        
    randL_MaxBranchPV: IntProperty(
        name = "Max Branch Amount Per Junction",
        description="The maximal amount of new branches one junction point CAN have.\nSet to 1 for no branches",
        default = 2,
        min = 1
        )
    
    randL_nXLimit: FloatProperty(
        name = "", 
        description="The individual branches do not extrude further in \nthe -X direction than this value from its starting point",
        default = -1.0,
    )
    
    randL_pXLimit: FloatProperty(
        name = "", 
        description="The individual branches do not extrude further in \nthe +X direction than this value from its starting point",
        default = 1.0,
    )
    
    randL_nYLimit: FloatProperty(
        name = "", 
        description="The individual branches do not extrude further in \nthe -Y direction than this value from its starting point",
        default = -1.0,
    )
    
    randL_pYLimit: FloatProperty(
        name = "", 
        description="The individual branches do not extrude further in \nthe +Y direction than this value from its starting point",
        default = 1.0,
    )
    
    randL_nZLimit: FloatProperty(
        name = "", 
        description="The individual branches do not extrude further in \nthe -Z direction than this value from its starting point",
        default = 0.5,
    )
    
    randL_pZLimit: FloatProperty(
        name = "", 
        description="The individual branches do not extrude further in \nthe +z direction than this value from its starting point",
        default = 1.0,
    )
    
    randL_enableAttachment: BoolProperty(
        name = "", 
        description="Check this box if you want to attach the tips of the lightning to an object",
        default = False,
    )
    
    lowPolyL_lookAtCamera: BoolProperty(
        name = "", 
        description="Check this box if you want the lightning plane to look continuously at a custom object (e.g. the camera) \nLeave it unchecked to generate a target object",
        default = False,
    )
    
    branchL_enableAttachment: BoolProperty(
        name = "", 
        description="Check this box if you want to attach the tips of the lightning to an object",
        default = False,
    )
    

class ElectrifyUI:
    bl_space_type = 'VIEW_3D'
    bl_region_type = 'UI'
    bl_category = 'Electrify'


#Simple Lightning UI --------------------------------------------------------------------------------------------------------------------------------------------------------------
class SimpleL_PT_UI(ElectrifyUI, Panel):
    bl_idname = "panel.simple"
    bl_label = "Simple Lightning" 
    bl_order = 0
    bl_options = {'DEFAULT_CLOSED'}

    def draw(self, context):
        layout = self.layout
        scene = context.scene
        randLProps = scene.randomL_props

        layout.operator('lightning.simple', text="Generate New", icon_value=pcoll["SIMPLE_ICON"].icon_id) 
        
        
#LowPoly Lightning UI --------------------------------------------------------------------------------------------------------------------------------------------------------------
class LowPolyL_PT_UI(ElectrifyUI, Panel):
    bl_idname = "panel.lowpoly"
    bl_label = "LowPoly Lightning" 
    bl_order = 1
    bl_options = {'DEFAULT_CLOSED'}

    def draw(self, context):
        layout = self.layout
        scene = context.scene
        lowPolyLProps = scene.lowPolyL_props
        
        row = layout.row()
        
        lowPolyLBox2 = layout.box()
        lookAtRow = lowPolyLBox2.row()
        lookAtRow.prop(lowPolyLProps, "lowPolyL_lookAtCamera")
        lookAtRow.label(text="Look at custom Object")
        targetRow = lowPolyLBox2.row()
        targetRow.prop_search(scene, "lowPoly_rand", context.scene, "objects",text="Target")
        
        if bpy.context.scene.lowPolyL_props.lowPolyL_lookAtCamera == True:
            targetRow.enabled = True
        else:
            targetRow.enabled = False
                
        if bpy.context.scene.lowPoly_rand == None and bpy.context.scene.lowPolyL_props.lowPolyL_lookAtCamera == True:
            lowPolyLBox2.label(text="No Object Selected!")
        
        row = layout.row()
        buttonRow = layout.row()
        
        if bpy.context.scene.lowPoly_rand == None and bpy.context.scene.lowPolyL_props.lowPolyL_lookAtCamera == True:
            buttonRow.operator('lightning.lowpoly', text="Generate New", icon_value=pcoll["CROSS_ICON"].icon_id)
            buttonRow.enabled = False
        else:
            buttonRow.operator('lightning.lowpoly', text="Generate New", icon_value=pcoll["LOWPOLY_ICON"].icon_id)
        
        
#Branched Lightning UI ------------------------------------------------------------------------------------------------------------------------------------------------------------
class BranchedL_PT_UI(ElectrifyUI, Panel):
    bl_idname = "panel.branched"
    bl_label = "Branched Lightning" 
    bl_order = 2 
    bl_options = {'DEFAULT_CLOSED'}

    def draw(self, context):
        layout = self.layout
        scene = context.scene
        randLProps = scene.randomL_props
        '''
        row = layout.row()
        
        branchLBox = layout.box()
        attachRow = branchLBox.row()
        attachRow.prop(randLProps, "branchL_enableAttachment")
        attachRow.label(text="Attach Lightning to Object")
        targetRow = branchLBox.row()
        targetRow.prop_search(scene, "attach_branch", context.scene, "objects",text="Target")
        
        if bpy.context.scene.randomL_props.branchL_enableAttachment == True:
            targetRow.enabled = True
        else:
            targetRow.enabled = False
                
        if bpy.context.scene.attach_branch == None and bpy.context.scene.randomL_props.branchL_enableAttachment == True:
            branchLBox.label(text="No Object Selected!")
        
        row = layout.row()
        buttonRow = layout.row()
        
        if bpy.context.scene.attach_branch == None and bpy.context.scene.randomL_props.branchL_enableAttachment == True:
            buttonRow.operator('lightning.branch', text="Generate New", icon_value=pcoll["CROSS_ICON"].icon_id)
            buttonRow.enabled = False
        else:
            buttonRow.operator('lightning.branch', text="Generate New", icon_value=pcoll["BRANCHED_ICON"].icon_id)
        '''
        layout.operator('lightning.branch', text="Generate New", icon_value=pcoll["BRANCHED_ICON"].icon_id)
   
   
#Random Lightning UI --------------------------------------------------------------------------------------------------------------------------------------------------------------
class RandomL_PT_UI(ElectrifyUI, Panel):
    bl_idname = "panel.random"
    bl_label = "Random Lightning"  
    bl_order = 3
    bl_options = {'DEFAULT_CLOSED'}

    def draw(self, context):
        layout = self.layout
        scene = context.scene
        randLProps = scene.randomL_props
        
        row = layout.row()
        
        layout.prop(randLProps, "randL_BranchAmount", text="Branch Amount")
        layout.prop(randLProps, "randL_MaxBranchPV", text="Max Branch Per Junction")
        
        row = layout.row()
        
        randLBox = layout.box()
        randLBox.label(text="Random Branch Extrusion Limits")
        rowX = randLBox.row()
        rowX.label(text="X Limits")
        rowX.prop(randLProps, "randL_nXLimit")
        rowX.prop(randLProps, "randL_pXLimit")
        
        rowY = randLBox.row()
        rowY.label(text="Y Limits")
        rowY.prop(randLProps, "randL_nYLimit")
        rowY.prop(randLProps, "randL_pYLimit")
        
        rowZ = randLBox.row()
        rowZ.label(text="Z Limits")
        rowZ.prop(randLProps, "randL_nZLimit")
        rowZ.prop(randLProps, "randL_pZLimit")
        
        row = layout.row()
        
        randLBox2 = layout.box()
        attachRow = randLBox2.row()
        attachRow.prop(randLProps, "randL_enableAttachment")
        attachRow.label(text="Attach Lightning to Object")
        targetRow = randLBox2.row()
        targetRow.prop_search(scene, "attach_rand", context.scene, "objects",text="Target")
        
        if bpy.context.scene.randomL_props.randL_enableAttachment == True:
            targetRow.enabled = True
        else:
            targetRow.enabled = False
                
        if bpy.context.scene.attach_rand == None and bpy.context.scene.randomL_props.randL_enableAttachment == True:
            randLBox2.label(text="No Object Selected!")
        
        row = layout.row()
        buttonRow = layout.row()
        
        if bpy.context.scene.attach_rand == None and bpy.context.scene.randomL_props.randL_enableAttachment == True:
            buttonRow.operator('lightning.random', text="Generate New", icon_value=pcoll["CROSS_ICON"].icon_id)
            buttonRow.enabled = False
        else:
            buttonRow.operator('lightning.random', text="Generate New", icon_value=pcoll["RANDOM_ICON"].icon_id)
        


#Registration----------------------------------------------------------------------------------------------------
classes = (
    ElectrifyProperties,
    BranchedL_PT_UI,
    SimpleL_PT_UI,
    RandomL_PT_UI,
    LowPolyL_PT_UI
)

def register():
    from bpy.utils import register_class
    for cls in classes:
        register_class(cls)

    bpy.types.Scene.randomL_props = PointerProperty(type=ElectrifyProperties)
    bpy.types.Scene.attach_rand = PointerProperty(type=bpy.types.Object)
    
    bpy.types.Scene.lowPolyL_props = PointerProperty(type=ElectrifyProperties)
    bpy.types.Scene.lowPoly_rand = PointerProperty(type=bpy.types.Object)
    
    bpy.types.Scene.attach_branch = PointerProperty(type=bpy.types.Object)

def unregister():
    from bpy.utils import unregister_class
    for cls in reversed(classes):
        unregister_class(cls)
    del bpy.types.Scene.randomL_props
    del bpy.types.Scene.lowPolyL_props
    del bpy.types.Scene.attach_rand
    del bpy.types.Scene.lowPoly_rand 
    del bpy.types.Scene.attach_branch
    